############################# PSQF 6272 Example 8b using R #####################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("Hmisc")) install.packages("Hmisc")
library(Hmisc) # To get bivariate correlations with p-values
# Note: has routine "describe" that competes with psych package

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("ordinal")) install.packages("ordinal")
library(ordinal) # To estimate MLMs for ordinal outcomes

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 8b            #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF6272/PSQF6272_Example8b/"
filename = "Example8b_Data.xlsx"
setwd(dir=filesave)

# Load Jonathan's custom R functions from folder within working directory
functions = paste0("R functions/",dir("R functions/"))
temp = lapply(X=functions, FUN=source)

#  Import trimmed example excel data file from sheet "grade10"
Example8b = read_excel(paste0(filesave,filename), sheet="grade10") 
# Convert to data frame to use in analysis
Example8b = as.data.frame(Example8b)

# Open external file to save results to (turn off to use console instead)
txtStart(file=paste0(filesave,"Example8b_R_Output.txt"))

# Labels for original variables in comments only
# districtID= "districtID: District ID number"
# studentID=  "studentID: Student ID number"
# schoolID=   "schoolID: School ID number"
# lunch=      "lunch: 0=Paid, 1=Reduced, 2=Free"
# math=       "math: Math Test Score"

print("R Descriptive Statistics within Student-Level Data")
prop.table(table(x=Example8b$lunch, useNA="ifany"))

# Create new dummy-coded predictors (start as blank, then recode)
Example8b$PvRF=NA; Example8b$PRvF=NA 
Example8b$PvRF[which(Example8b$lunch==0)]=0  # Replace each for paid lunch
Example8b$PRvF[which(Example8b$lunch==0)]=0
Example8b$PvRF[which(Example8b$lunch==1)]=1  # Replace each for reduced
Example8b$PRvF[which(Example8b$lunch==1)]=0
Example8b$PvRF[which(Example8b$lunch==2)]=1  # Replace each for free lunch
Example8b$PRvF[which(Example8b$lunch==2)]=1
# PvRF= "PvRF: 0=Paid, 1=Reduced or Free Lunch"
# PRvF= "PRvF: 0=Paid or Reduced, 1=Free Lunch"

# Filter to only cases complete on all variables to be used below (before cluster means)
Example8b = Example8b[complete.cases(Example8b[ , c("math","lunch")]),]

# Compute level-2 school means for level-1 variables using Jonathan's function
Example8b = addUnitMeans(data=Example8b, unitVariable="schoolID",
                         meanVariables=c("math","PvRF","PRvF"), 
                         newNames=c("SM_math","SM_PvRF","SM_PRvF"))

print("R Descriptive Statistics within School-Level-2 Data")
schoolMeans = unique(Example8b[,c("schoolID","NperschoolID","SM_math","SM_PvRF","SM_PRvF")])
describe(x=schoolMeans[ , c("NperschoolID","SM_math","SM_PvRF","SM_PRvF")])

# Compute level-3 district means for level-1 variables using Jonathan's function
Example8b = addUnitMeans(data=Example8b, unitVariable="districtID",
                         meanVariables=c("math","PvRF","PRvF"), 
                         newNames=c("DM_math","DM_PvRF","DM_PRvF"))

print("R Descriptive Statistics within District-Level-3 Data")
districtMeans = unique(Example8b[,c("districtID","NperdistrictID","DM_math","DM_PvRF","DM_PRvF")])
describe(x=districtMeans[ , c("NperdistrictID","DM_math","DM_PvRF","DM_PRvF")])

# Cluster-mean-center level-1 variables at level-2 means
Example8b$WS_PvRF = Example8b$PvRF - Example8b$SM_PvRF
Example8b$WS_PRvF = Example8b$PRvF - Example8b$SM_PRvF
Example8b$WS_math = Example8b$math - Example8b$SM_math 
# WS_PvRF= "WS_PvRF: Within-School Paid vs Reduced/Free Lunch"
# WS_PRvF= "WS_PRvF: Within-School Paid/Reduced vs Free Lunch"
# WS_math= "WS_math: Within-School Math"

# Cluster-mean-center level-2 variables at level-3 means
Example8b$WD_PvRF = Example8b$SM_PvRF - Example8b$DM_PvRF
Example8b$WD_PRvF = Example8b$SM_PRvF - Example8b$DM_PRvF
Example8b$WD_math = Example8b$SM_math - Example8b$DM_math
# WD_PvRF= "WD_PvRF: Within-District Paid vs Reduced/Free Lunch"
# WD_PRvF= "WD_PRvF: Within-District Paid/Reduced vs Free Lunch"
# WD_math= "WD_math: Within-School Math"

# Center cluster means so ref school has 0=.72, 1=.09, and 2=.28
Example8b$DM_PvRF28 = Example8b$DM_PvRF - .28
Example8b$DM_PRvF19 = Example8b$DM_PRvF - .19

print("R Descriptive Statistics within Student-Level-1 Data")
describe(x=Example8b[ , c("WS_PvRF","WS_PRvF","WD_PvRF","WD_PRvF","DM_PvRF","DM_PRvF")])
rcorr(x=as.matrix(Example8b[,c("math","SM_math","DM_math","WS_math","WD_math")]), type="pearson")


################################################################################
#####                 BEGIN MODELS FOR EXAMPLE 8b                          #####
################################################################################

# Empty models

print("R Model 1a: Empty Means, Two-Level Random Intercept for Math")
Model1a = lmer(data=Example8b, REML=TRUE, formula=math~1+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1a, chkREML=FALSE); summary(Model1a, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT")
icc(Model1a); ranova(Model1a)

txtComment(" ") # insert blank space in output

###############################################################################
print("R Model 1b: Empty Means, Three-Level Random Intercept for Math")
Model1b = lmer(data=Example8b, REML=TRUE, 
               formula=math~1+(1|districtID)+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1b, chkREML=FALSE); summary(Model1b, ddf="Satterthwaite")
print("Show proportions of variance per level and random intercept LRTs")
icc(Model1b, by_group=TRUE); icc(Model1b); ranova(Model1b)

print("Show saved variances from model 1b")
as.data.frame(VarCorr(Model1b))

# Compute total variance from model 1b saved variances
total1b = as.data.frame(VarCorr(Model1b))[1,4] +
          as.data.frame(VarCorr(Model1b))[2,4] +
          as.data.frame(VarCorr(Model1b))[3,4]

print("L2 ICC for Children in Schools")
 (as.data.frame(VarCorr(Model1b))[1,4] +
  as.data.frame(VarCorr(Model1b))[2,4]) / total1b

print("L3 ICC for Schools in Districts")
 as.data.frame(VarCorr(Model1b))[2,4] / (as.data.frame(VarCorr(Model1b))[1,4] +
                                         as.data.frame(VarCorr(Model1b))[2,4])

txtComment(" ") # insert blank space in output

###############################################################################
print("R: Empty Means, Random Intercept for Ordinal Lunch using ordinal package")
Model1c = clmm(data=Example8b, link="logit", 
               formula=as.factor(lunch)~1+(1|districtID)+(1|schoolID))
print("Show results with -2LL, provides submodel thresholds (multiply by -1 to get intercepts)")
-2*logLik(Model1c); summary(Model1c)
print("ICC using pi^2/3 = 3.29 as L1 residual variance") 
icc(Model1c, by_group=TRUE); icc(Model1c) 

print("Show saved variances from model 1c")
as.data.frame(VarCorr(Model1c))

# Compute total variance from model 1c saved variances
total1c = as.data.frame(VarCorr(Model1c))[1,1] +
          as.data.frame(VarCorr(Model1c))[1,2] + 3.29

print("L2 ICC for Children in Schools")
 (as.data.frame(VarCorr(Model1c))[1,1] +
  as.data.frame(VarCorr(Model1c))[1,2]) / total1c

print("L3 ICC for Schools in Districts")
as.data.frame(VarCorr(Model1c))[1,2] / (as.data.frame(VarCorr(Model1c))[1,1] +
                                        as.data.frame(VarCorr(Model1c))[1,2])

txtComment(" ") # insert blank space in output


##################################################################################

# Add Paid vs Reduced/Free Lunch predictor slopes

print("R Model 2a: Add Add Variable-Centered Lunch Predictors at all 3 Levels")
Model2a = lmer(data=Example8b, REML=TRUE, formula=math~1+WS_PvRF+WS_PRvF+WD_PvRF+WD_PRvF
                 +DM_PvRF28+DM_PRvF19+(1|districtID)+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2a, chkREML=FALSE); summary(Model2a, ddf="Satterthwaite")

print("L2 PvRF Context"); contest1D(Model2a, L=c(0,-1, 0, 1, 0, 0, 0))
print("L2 PRvF Context"); contest1D(Model2a, L=c(0, 0,-1, 0, 1, 0, 0))
print("L3 PvRF Context"); contest1D(Model2a, L=c(0, 0, 0,-1, 0, 1, 0))
print("L3 PRvF Context"); contest1D(Model2a, L=c(0, 0, 0, 0,-1, 0, 1))

txtComment(" ") # insert blank space in output


print("R Model 2b: Add L2 Random Slope for Cluster-MC WS_PvRF")
Model2b = lmer(data=Example8b, REML=TRUE, formula=math~1+WS_PvRF+WS_PRvF+WD_PvRF+WD_PRvF
               +DM_PvRF28+DM_PRvF19+(1|districtID)+(1+WS_PvRF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2b, chkREML=FALSE); summary(Model2b, ddf="Satterthwaite")
print("LRT for L2 random slope"); ranova(Model2b)

txtComment(" ") # insert blank space in output


print("R Model 2c: Add L3 Random Slope for Cluster-MC WS_PvRF")
Model2c = lmer(data=Example8b, REML=TRUE, formula=math~1+WS_PvRF+WS_PRvF+WD_PvRF+WD_PRvF
               +DM_PvRF28+DM_PRvF19+(1+WS_PvRF|districtID)+(1+WS_PvRF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2c, chkREML=FALSE); summary(Model2c, ddf="Satterthwaite")
print("LRT for L3 random slope"); ranova(Model2c)

txtComment(" ") # insert blank space in output


#######################################################################################
# Close external file of saved results
txtStop()



