* Output options: all can be turned on/off by adding or removing the NO;
* page number, date, centering, or page breaks, page length and width;
OPTIONS NOnumber NOdate NOcenter FormDlim=" " PageSize=MAX LineSize=MAX SPOOL;
* Eliminate SAS default titles and names of tables in output (TRACE ON to show);
TITLE; ODS TRACE OFF; *ODS LISTING;

***********************************************************************************;
*******     MACRO PROGRAMS TO AUTOMATE CALCULATIONS FOR THIS EXAMPLE        *******;
*******               NOTHING IN HERE NEEDS TO BE CHANGED                   *******;
***********************************************************************************;

* To use TotalR2 macro;
* DV =        Case-sensitive name of dependent variable (required);
* PredFewer = Name of OUTPM= data file of predicted outcomes for first model (required);
* PredMore =  Name of OUTPM= data file of predicted outcomes for second model (optional);
%MACRO TotalR2(DV=, Pred1=, Pred2=);
PROC CORR NOPRINT NOSIMPLE DATA=&Pred1. OUTP=Corr1; VAR pred &DV.; RUN;
DATA Corr1; LENGTH Name $30.; SET Corr1; Name="&Pred1.";
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
PROC CORR NOPRINT NOSIMPLE DATA=&Pred2. OUTP=Corr2; VAR pred &DV.; RUN;
DATA Corr2; LENGTH Name $30.; SET Corr2; Name="&Pred2."; 
     IF Pred=. THEN Pred=0; PredCorr=Pred; TotalR2=PredCorr*PredCorr; 
     IF _NAME_="Pred" OR MISSING(_NAME_)=1 THEN DELETE; 
     KEEP Name PredCorr TotalR2; RUN;
DATA CorrCompare; LENGTH Name $30.; SET Corr1 Corr2; 
     TotalR2Diff=TotalR2-LAG1(TotalR2);
     KEEP Name PredCorr TotalR2 TotalR2Diff; RUN; 
%END;
%IF %LENGTH(&Pred2.)>0 %THEN %LET ToPrint=CorrCompare; 
     %ELSE %LET ToPrint=Corr1;
TITLE8 "Total-R2 Per Model";
%IF %LENGTH(&Pred2.)>0 %THEN %DO;
TITLE9 "Change in Total-R2 for &Pred1. vs. &Pred2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;   
%MEND TotalR2;

* To use PseudoR2 macro;
* Ncov =     TOTAL # entries in covariance parameter estimates table;
* CovFewer = Name of ODS CovParms table for nested baseline model (required);
* CovMore1 = Name of ODS CovParms table for comparison model1 (required);
* CovMore2 = Name of ODS CovParms table for comparison model2 (optional);
%MACRO PseudoR2(NCov=, CovFewer=, CovMore1=, CovMore2=);
DATA &CovFewer.; LENGTH Name $30.; SET &CovFewer.; Name="&CovFewer."; RUN;
DATA &CovMore1.; LENGTH Name $30.; SET &CovMore1.; Name="&CovMore1."; RUN;
DATA CovCompare1; LENGTH Name $30.; SET &CovFewer. &CovMore1.; RUN;
DATA CovCompare1; SET CovCompare1; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
%IF %LENGTH(&CovMore2.)>0 %THEN %DO; %LET lagchange=%EVAL(&Ncov.*2);
DATA &CovMore2.; LENGTH Name $30.; SET &CovMore2.; Name="&CovMore2."; RUN;
DATA CovCompare2; LENGTH Name $30.; SET &CovFewer. &CovMore2.; RUN;
DATA CovCompare2; SET CovCompare2; 
     PseudoR2=(LAG&Ncov.(Estimate)-Estimate)/LAG&Ncov.(Estimate); RUN;
DATA CovCompare; SET CovCompare1 CovCompare2;
     PseudoR2Change=PseudoR2-(LAG&lagchange(PseudoR2)); RUN;
%END; %IF %LENGTH(&CovMore2.)>0 %THEN %LET ToPrint=CovCompare; 
     %ELSE %LET ToPrint=CovCompare1;
DATA &ToPrint.; SET &ToPrint.; 
     IF CovParm IN("UN(2,1)","UN(3,1)","UN(3,2)","UN(4,1)","UN(4,2)","UN(4,3)",
                   "UN(5,1)","UN(5,2)","UN(5,3)","UN(5,4)",
                   "UN(6,1)","UN(6,2)","UN(6,3)","UN(6,4)","UN(6,5)") THEN DELETE; 
     DROP Zvalue ProbZ; RUN;
TITLE8 "Pseudo-R2 Relative to &CovFewer.";
%IF %LENGTH(&CovMore2.)>0 %THEN %DO;
TITLE9 "Change in Pseudo-R2 for &CovMore1. vs. &CovMore2."; %END;
PROC PRINT NOOBS DATA=&ToPrint.; RUN; TITLE8; TITLE9;
%MEND PseudoR2;

* To use FitTest macro;
* FitFewer = name of infocrit table for nested model;
* FitMore  = name of infocrit table for comparison model;
%MACRO FitTest(FitFewer=, FitMore=);
DATA &FitFewer.; LENGTH Name $30.; SET &FitFewer.; Name="&FitFewer."; RUN;
DATA &FitMore.;  LENGTH Name $30.; SET &FitMore.;  Name="&FitMore.";  RUN;
DATA FitCompare; LENGTH Name $30.; SET &FitFewer. &FitMore.; RUN;
DATA FitCompare; SET FitCompare; DevDiff=Lag1(Neg2LogLike)-Neg2LogLike;
     DFdiff=Parms-LAG1(Parms); Pvalue=1-PROBCHI(DevDiff,DFdiff);
     DROP AICC HQIC CAIC; RUN;
TITLE2 "Likelihood Ratio Test for &FitFewer. vs. &FitMore.";
PROC PRINT NOOBS DATA=FitCompare; RUN; TITLE2;
%MEND FitTest;


***********************************************************************************;
*******           BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 8b          *******;
*******               CHANGE "filesave" TO YOUR FOLDER PATH                 *******;
***********************************************************************************;

* Define global variable for file location to be replaced in code below;
* \\Client\ precedes actual path when using UIowa Virtual Desktop;
%LET filesave=C:\Dropbox\23_PSQF6272\PSQF6272_Example8b;

* Import trimmed example data excel data file from sheet "grade10" into SAS;
PROC IMPORT DATAFILE="&filesave.\Example8b_Data.xlsx" OUT=work.Example8b DBMS=XLSX REPLACE; 
     SHEET="grade10"; GETNAMES=YES; RUN;

* Open external file to save results to;
ODS RTF FILE="&filesave.\Example8b_SAS_Output.rtf" 
        STYLE=HTMLBlue STARTPAGE=NO BODYTITLE;

* Data preparation;
DATA work.Example8b; SET work.Example8b; 
* Add labels to original variables;
  LABEL districtID= "districtID: District ID number"
        studentID=  "studentID: Student ID number"
        schoolID=   "schoolID: School ID number"
        lunch=      "lunch: 0=Paid, 1=Reduced, 2=Free"
        math=       "math: Math Test Score";
* Create new dummy-coded predictors; 
       IF lunch=0 THEN DO; PvRF=0; PRvF=0; END;
  ELSE IF lunch=1 THEN DO; PvRF=1; PRvF=0; END;
  ELSE IF lunch=2 THEN DO; PvRF=1; PRvF=1; END;
  LABEL PvRF=   "PvRF: 0=Paid, 1=Reduced or Free Lunch"
        PRvF=   "PRvF: 0=Paid or Reduced, 1=Free Lunch"; 
* Filter to complete cases before computing cluster means;
  IF NMISS(math,lunch)>0 THEN DELETE;
RUN;

* Create level-2 school means for level-1 variables;
PROC SORT DATA=work.Example8b; BY schoolID; RUN;
PROC MEANS NOPRINT DATA=work.Example8b; BY schoolID;
     VAR math PvRF PRvF;
     OUTPUT OUT=work.SchoolMeans 
          MEAN(math PvRF PRvF)= SM_math SM_PvRF SM_PRvF; 
RUN;

* Label new school means;
DATA work.SchoolMeans; SET work.SchoolMeans;
     schoolN=_FREQ_; DROP _TYPE_ _FREQ_;
     LABEL schoolN=   "schoolN: # Students Sampled Per School"
           SM_math=   "SM_math: School Mean Math"
           SM_PvRF=   "SM_PvRF: School Mean 0=No, 1=Reduced or Free Lunch"
           SM_PRvF=   "SM_PRvF: School Mean 0=Paid or Reduced, 1=Free Lunch";
RUN;

TITLE "SAS Descriptive Statistics within School-Level-2 Data";
PROC MEANS NDEC=2 DATA=work.SchoolMeans; 
     VAR schoolN SM_math SM_PvRF SM_PRvF; RUN;
TITLE;

* Merge school means back into student data;
PROC SORT DATA=work.Example8b; BY schoolID; RUN;
DATA work.Example8b; MERGE work.Example8b work.SchoolMeans; BY schoolID; RUN;

* Create level-3 district means for level-1 variables;
PROC SORT DATA=work.Example8b; BY districtID; RUN;
PROC MEANS NOPRINT DATA=work.Example8b; BY districtID;
     VAR math PvRF PRvF;
     OUTPUT OUT=work.DistrictMeans 
          MEAN(math PvRF PRvF)= DM_math DM_PvRF DM_PRvF; 
RUN;

* Label new district means;
DATA work.DistrictMeans; SET work.DistrictMeans;
     districtN=_FREQ_; DROP _TYPE_ _FREQ_;
     LABEL districtN= "districtN: # Students Sampled Per District"
           DM_math=   "DM_math: District Mean of Student Math"
           DM_PvRF=   "DM_PvRF: District Mean 0=No, 1=Reduced or Free Lunch"
           DM_PRvF=   "DM_PRvF: District Mean 0=Paid or Reduced, 1=Free Lunch";
RUN;

* Get # schools per district and district mean of school means;
PROC SORT DATA=work.Example8b; BY districtID schoolID; RUN;
PROC MEANS NOPRINT DATA=work.Example8b; BY districtID schoolID;
     VAR math;
     OUTPUT OUT=work.DistrictSchools N(math)= districtNschools; 
RUN;
PROC MEANS NOPRINT DATA=work.DistrictSchools; BY districtID;
     VAR districtNschools; 
     OUTPUT OUT=work.DistrictSchools N(districtNschools)= districtNschools; 
RUN;

* Merge with district means and label new variable;
DATA work.DistrictMeans; MERGE work.DistrictMeans work.DistrictSchools; BY districtID; 
     DROP _TYPE_ _FREQ_;
     LABEL districtNschools= "districtNschools: # Schools Sampled Per District";
RUN;

TITLE "SAS Descriptive Statistics within District-Level-3 Data";
PROC MEANS NDEC=2 DATA=work.DistrictMeans; 
     VAR districtN districtNschools DM_math DM_PvRF DM_PRvF; RUN;
PROC FREQ DATA=work.DistrictMeans; TABLE districtNschools; RUN;
TITLE; 

* Merge district means back into student data;
PROC SORT DATA=work.Example8b; BY districtID; RUN;
DATA work.Example8b; MERGE work.Example8b work.DistrictMeans; BY districtID; RUN;

* Center predictors;
DATA work.Example8b; SET work.Example8b;
* Center district means so ref school has 0=.70, 1=.11, and 2=.19;
  DM_PvRF28 = DM_PvRF - .28;
  DM_PRvF19 = DM_PRvF - .19;  
* Cluster-mean-center level-1 variables;
  WS_PvRF = PvRF - SM_PvRF;
  WS_PRvF = PRvF - SM_PRvF;
  WS_math = math - SM_math;
LABEL WS_PvRF= "WS_PvRF: Within-School Paid vs Reduced/Free Lunch"
      WS_PRvF= "WS_PRvF: Within-School Paid/Reduced vs Free Lunch"
      WS_math= "WS_math: Within-School Math";
* Cluster-mean-center level-2 variables;
  WD_PvRF = SM_PvRF - DM_PvRF;
  WD_PRvF = SM_PRvF - DM_PRvF;
  WD_math = SM_math - DM_math;
LABEL WD_PvRF= "WD_PvRF: Within-District Paid vs Reduced/Free Lunch"
      WD_PRvF= "WD_PRvF: Within-District Paid/Reduced vs Free Lunch"
      WD_math= "WD_math: Within-District Math";
RUN;

TITLE "SAS Descriptive Statistics within Student-Level-1 Data";
PROC MEANS NOLABELS NDEC=3 N MEAN VAR STDDEV MIN MAX DATA=work.Example8b; 
     VAR WS_PvRF WS_PRvF WD_PvRF WD_PRvF; RUN; TITLE;
PROC CORR NOSIMPLE DATA=work.Example8b; 
     VAR math SM_math DM_math WS_math WD_math; RUN;


***********************************************************************************;
*******                    BEGIN MODELS FOR EXAMPLE 8b                      *******;
***********************************************************************************;

* Empty Models;

TITLE "SAS Model 1a: Empty Means, Two-Level Random Intercept for Math";
PROC MIXED DATA=work.Example8b NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS schoolID; 
     MODEL math = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / VCORR TYPE=UN SUBJECT=schoolID;  * VCORR gives ICC for L2 schools;
     ODS OUTPUT InfoCrit=FitEmpty1a CovParms=CovEmpty1a; * Save fit and variances; 
RUN; TITLE;

TITLE "SAS Model 1b: Empty Means, Three-Level Random Intercept for Math";
PROC MIXED DATA=work.Example8b NOCLPRINT COVTEST NAMELEN=100 IC METHOD=REML;
     CLASS districtID schoolID; 
     MODEL math = / SOLUTION DDFM=Satterthwaite; 
     RANDOM INTERCEPT / TYPE=UN SUBJECT=districtID;      * L3 districts;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID;        * L2 schools;
     *RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID(district); * This version is slower but gives same result;
     ODS OUTPUT InfoCrit=FitEmpty1b CovParms=CovEmpty1b; * Save fit and variances; 
     *PARMS (17) (22) (253); * Adding start values for variances in order of output;
RUN; TITLE;
* Calculate difference in model fit relative to model 1a;
%FitTest(FitFewer=FitEmpty1a, FitMore=FitEmpty1b);

* Compute proportion of variance of each kind;
DATA work.PropVar; LENGTH CovParm $12; SET work.CovEmpty1b; 
     IF CMISS(Subject)=0 THEN CovParm=Subject; RUN; 
PROC TRANSPOSE DATA=work.PropVar OUT=work.Prop; ID CovParm; VAR Estimate; RUN; 
DATA work.Prop; SET work.Prop; 
     Total = SUM(districtID, schoolID, Residual); 
     PropL3District = districtID/Total; 
     PropL2School = schoolID/Total; 
     PropL1Resid = Residual/Total; 
     ICCL2 = (PropL3District+PropL2School);
     ICCL3 = PropL3District / (PropL3District+PropL2School); DROP _NAME_; RUN;
TITLE "Empty Model Proportions of Variance";
PROC PRINT NOOBS DATA=work.Prop; RUN; TITLE;

TITLE "SAS: Empty Means, Three-Level Random Intercept for Ordinal Student Lunch";
PROC GLIMMIX DATA=work.Example8b NOCLPRINT NAMELEN=100 GRADIENT METHOD=LAPLACE;
     CLASS schoolID; 
     MODEL lunch (DESCENDING) = / SOLUTION LINK=CLOGIT DIST=MULT DDFM=NONE; 
     RANDOM INTERCEPT / TYPE=UN SUBJECT=districtID;      * L3 districts;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID;        * L2 schools;
     ODS OUTPUT CovParms=CovEmptyLunch;
RUN; TITLE;

* Compute proportion of variance of each kind;
PROC TRANSPOSE DATA=work.CovEmptyLunch OUT=work.PropLunch; ID Subject; VAR Estimate; RUN; 
DATA work.PropLunch; SET work.PropLunch; 
     Total = SUM(districtID, schoolID, 3.29); 
     PropL3District = districtID/Total; 
     PropL2School = schoolID/Total; 
     PropL1Resid = 3.29/Total; 
     ICCL2 = (PropL3District+PropL2School);
     ICCL3 = PropL3District / (PropL3District+PropL2School); DROP _NAME_; RUN;
TITLE "Empty Model Proportions of Variance";
PROC PRINT NOOBS DATA=work.PropLunch; RUN; TITLE;

***********************************************************************************;
* Add Paid vs Reduced/Free Lunch predictor slopes;

TITLE "SAS HPMIXED Model 2a: Add Variable-Centered Lunch Predictors at all 3 Levels";
PROC HPMIXED DATA=work.Example8b NOCLPRINT NAMELEN=100 METHOD=REML;
     CLASS schoolID districtID;         
     MODEL math = WS_PvRF WS_PRvF WD_PvRF WD_PRvF DM_PvRF28 DM_PRvF19 
                  / SOLUTION DDFM=RESIDUAL;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=districtID; * L3 districts;
     RANDOM INTERCEPT / TYPE=UN SUBJECT=schoolID;   * L2 schools;
     ODS OUTPUT InfoCrit=Fit2a CovParms=Cov2a; * Save fit and variances; 
     ESTIMATE "L2 PvRF context" WS_PvRF -1 WD_PvRF   1;
     ESTIMATE "L2 PRvF context" WS_PRvF -1 WD_PRvF   1;
     ESTIMATE "L3 PvRF context" WD_PvRF -1 DM_PvRF28 1;
     ESTIMATE "L3 PRvF context" WD_PRvF -1 DM_PRvF19 1;
RUN; TITLE;
* Pseudo-R2 relative to empty means, random intercept model;
%PseudoR2(NCov=3, CovFewer=CovEmpty1b, CovMore1=Cov2a);

TITLE "SAS HPMIXED Model 2b: Add L2 Random Slope for Cluster-MC WS_PvRF";
PROC HPMIXED DATA=work.Example8b NOCLPRINT NAMELEN=100 METHOD=REML;
     CLASS schoolID districtID;         
     MODEL math = WS_PvRF WS_PRvF WD_PvRF WD_PRvF DM_PvRF28 DM_PRvF19 
                  / SOLUTION DDFM=RESIDUAL; 
     RANDOM INTERCEPT         / TYPE=UN SUBJECT=districtID;   * L3 districts;
     RANDOM INTERCEPT WS_PvRF / TYPE=UN SUBJECT=schoolID;    * L2 schools;
RUN; TITLE;

TITLE "SAS HPMIXED Model 2c: Add L3 Random Slope for Cluster-MC WS_PvRF";
PROC HPMIXED DATA=work.Example8b NOCLPRINT NAMELEN=100 METHOD=REML;
     CLASS schoolID districtID;         
     MODEL math = WS_PvRF WS_PRvF WD_PvRF WD_PRvF DM_PvRF28 DM_PRvF19 
                  / SOLUTION DDFM=RESIDUAL;  * Had to make nesting explicit to avoid error;
     RANDOM INTERCEPT WS_PvRF / TYPE=UN SUBJECT=districtID;  * L3 districts;
     RANDOM INTERCEPT WS_PvRF / TYPE=UN SUBJECT=schoolID;    * L2 schools;
RUN;

***********************************************************************************;
* Close external file of saved results;
ODS RTF CLOSE;
